/* ExApplication
 */
#include <stdio.h>
#include <assert.h>
#include <malloc.h>
#include <be/app/Roster.h>
#include <be/storage/AppFileInfo.h>
#include <be/storage/FindDirectory.h>
#include <be/storage/Path.h>
#include <be/storage/Directory.h>
#include <be/storage/Mime.h>
#include <be/support/DataIO.h>
//#include "ArpKernel/ArpAboutWindow.h"
//#include "ArpKernel/ArpDebug.h"
#include "LpApplication.h"
#include "LpEntryWindow.h"
#include "LpWindowStateI.h"
#include "LpXmlStream.h"

const char* LpAppSig = "application/x-vnd.ARP-LesserPasswords";

// This directory will be located in the user settings directory.
#define STR_ARPSETTINGSDIR		"AngryRedPlanet"
// This directory will be located in STR_ARPSETTINGSDIR.
#define STR_LPSETTINGSDIR	"LesserPasswords"

#define STR_STATE_FILE		"state"

/***************************************************************************
 * LP-APPLICATION
 ***************************************************************************/
LpApplication::LpApplication()
		: BApplication(LpAppSig),
		mIsQuitting(false)
{
	ReadState();
	ShowEntryWin();
}

LpApplication::~LpApplication()
{
}

void LpApplication::MessageReceived(BMessage *message)
{
	switch (message->what) {
		default:
			inherited::MessageReceived(message);
			break;
	}
}

bool LpApplication::QuitRequested(void)
{
	mIsQuitting = true;
	SaveState();
	bool	answer = inherited::QuitRequested();
	if (answer == false) mIsQuitting = false;
	return answer;
}

bool LpApplication::IsQuitting() const
{
	return mIsQuitting;
}

status_t LpApplication::SaveState()
{
	BFile		*file;
	BWindow		*win;
	file = SettingsFile(B_WRITE_ONLY | B_CREATE_FILE | B_ERASE_FILE,
						STR_STATE_FILE);
	if (file == 0) return B_ERROR;
	LpXmlStream	stream(file);
		
	int32		k = 0;
	string_vec	keys, values;
	keys.push_back("version");
	values.push_back("1.0");
	stream.WriteStartTag("State", &keys, &values);
	stream.WriteCR();
	stream.PushTab();
	while ( (win = be_app->WindowAt(k)) != 0 ) {
		LpWindowStateI*	state = dynamic_cast<LpWindowStateI*>(win);
		if (state != 0) state->WriteState(stream);
		k++;
	}
	stream.PopTab();
	stream.WriteEndTag("State");

	delete file;
	return B_OK;
}

void LpApplication::ReadState()
{
	BFile			*file;
	file = SettingsFile(B_READ_ONLY, STR_STATE_FILE);
	if (file == 0) return;
	LpXmlStream		stream(file);
	const char		*tag;
	string_vec	keys, values;
	while ((tag = stream.ReadNextTag(&keys, &values)) != 0) {
		if (strcmp(tag, ARPSTR_WINDOW_XML_TAG) == 0)
			ReadWindowState(stream, keys, values);
		FreeVector(keys);
		FreeVector(values);
	}
	delete file;
}

void LpApplication::ReadWindowState(LpXmlStreamI& stream,
									string_vec& keys,
									string_vec& values)
{
	int32		index = IndexFor(ARPSTR_TYPE, keys);
	if (index < 0) return;
	char		*type = values[index];
	if (type == 0) return;
	BWindow		*win = 0;
	if (strcmp(type, ARPSTR_LPENTRYWINDOW) == 0) {
		win = new LpEntryWindow(BRect(80, 80, 440, 380));
		mEntryWin = BMessenger(win);
	}
	LpWindowStateI*	state = dynamic_cast<LpWindowStateI*>(win);
	if (state != 0) {
		state->ReadState(stream, ARPSTR_WINDOW_XML_TAG, type);
	}
	if (win != 0) win->Show();
}

void LpApplication::AboutRequested(void)
{
#if 0
	if( !mAboutWin.IsValid() ) {
		ArpAboutWindow* win = 
			new ArpAboutWindow(0, "Lesser Passwords", PROGRAM_VERSION, __DATE__,
					"Lesser Passwords version " PROGRAM_VERSION
					" / " __DATE__ "\n"
					B_UTF8_COPYRIGHT "2000 Angry Red Planet, Inc.\n\n"
					
					"A simple freeware utility for storing unencrypted\n"
					"passwords.\n\n"
					
					"For more info and the latest version, see\n"
					"<URL:http://www.angryredplanet.com/>.\n\n"
					
					"Written by Eric Hackborn\n"
					"hackborn@angryredplanet.com\n\n");

		mAboutWin = BMessenger(win);
		win->Show();
	}
#endif
}

void LpApplication::ShowEntryWin(void)
{
	if( !mEntryWin.IsValid() ) {
		LpEntryWindow	*win = new LpEntryWindow(BRect(80, 80, 440, 380));
		mEntryWin = BMessenger(win);
		win->Show();
	} else {
		BHandler	*target;
		BLooper		*looper;
		if ((target = mEntryWin.Target(&looper)) != 0)
			((BWindow*)target)->Activate(true);
	}
}

BFile* LpApplication::SettingsFile(	uint32 open_mode,
									const char *fileName,
									const char *mimeType)
{
	if (VerifySettingsDirectory() == false) return 0;

	BPath	path;
	find_directory(B_USER_SETTINGS_DIRECTORY, &path);
	if (path.InitCheck() != B_OK) return 0;

	char	str[strlen(path.Path())
					+ strlen(STR_ARPSETTINGSDIR)
					+ strlen(STR_LPSETTINGSDIR)
					+ strlen(fileName)
					+ 10];
	sprintf(str, "%s/%s/%s/%s",
			path.Path(),
			STR_ARPSETTINGSDIR,
			STR_LPSETTINGSDIR,
			fileName);
	BPath	path2(str);
	if (path2.InitCheck() != B_OK) return 0;
	
	BFile	*file = new BFile(path2.Path(), open_mode);
	if (file == 0) return 0;
	if (file->InitCheck() != B_OK) {
		delete file;
		return 0;
	}

	if (open_mode&B_CREATE_FILE) {
		// Set the new file's MIME type
		BNodeInfo	nodeInfo(file);
		nodeInfo.SetType(mimeType);
	}
	
	return file;
}

bool LpApplication::VerifySettingsDirectory()
{
	BPath		path;
	find_directory(B_USER_SETTINGS_DIRECTORY, &path);
	if (path.InitCheck() != B_OK) return false;
	
	BDirectory	settingsDir(path.Path());
	if (settingsDir.InitCheck() != B_OK) return false;
	if (settingsDir.Contains(STR_ARPSETTINGSDIR, B_DIRECTORY_NODE) == false) {
		if (settingsDir.CreateDirectory(STR_ARPSETTINGSDIR, 0) != B_OK)
			return false;
	}
	BPath		arpPath(path.Path(), STR_ARPSETTINGSDIR);
	if (arpPath.InitCheck() != B_OK) return false;
	BDirectory	arpDir(arpPath.Path());
	if (arpDir.InitCheck() != B_OK) return false;
	
	if (arpDir.Contains(STR_LPSETTINGSDIR, B_DIRECTORY_NODE) == false) {
		if (arpDir.CreateDirectory(STR_LPSETTINGSDIR, 0) != B_OK)
			return false;
	}

	BPath		devicePath(arpPath.Path(), STR_LPSETTINGSDIR);
	if (devicePath.InitCheck() != B_OK) return false;
	return true;
}
