/* LpEntry.h
 * Copyright (c)2000 by Eric Hackborn.
 * All rights reserved.
 *
 * This code is not public domain, nor freely distributable.
 * Please direct any questions or requests to Eric Hackborn,
 * at <hackborn@angryredplanet.com>.
 *
 * ----------------------------------------------------------------------
 *
 * Known Bugs
 * ~~~~~~~~~~
 *
 *	- None!  Ha ha!
 *
 * ----------------------------------------------------------------------
 *
 * History
 * ~~~~~~~
 * 04.18.00		hackborn
 * Created this file.
 */

#ifndef LP_LPENTRY_H
#define LP_LPENTRY_H

#include <vector.h>
#include <be/app/Message.h>
#include "LpXmlStreamI.h"
#include "string_vec.h"

// This is the message what that is used for LpEntries that have been
// converted to BMessages.
#define ARPMSG_ENTRY	'etry'

/*************************************************************************
 * LP-ENTRY
 * This class defines a single entry in the LesserPasswords app.  An
 * entry is basically a series of key / value pairs with a little extra
 * information.
 *************************************************************************/
class LpEntry
{
public:
	LpEntry(); 
	/* Create a new entry populated with the data in entryMsg.  This
	 * message must have a what of ARPMSG_ENTRY and be in the format
	 * used by the AsMessage() method of this class.
	 */
	LpEntry(const BMessage *entryMsg); 
	virtual ~LpEntry();

	const char* Name() const;
	void SetName(const char* name);
	const char* Url() const;
	void SetUrl(const char* url);
	/* Write the key and value at index into key and value, respectively.
	 * Answer with B_OK if the key and value existed and everything's fine,
	 * otherwise B_ERROR.
	 */
	status_t KeyValueAt(uint32 index,
						const char** key,
						const char** value) const;
	/* Add the supplied key and value to myself.  Answer B_OK if
	 * everything went fine, B_NO_MEMORY for no memory, otherwise
	 * B_ERROR.
	 */
	status_t AddKeyValue(	const char* key,
							const char* value);
	/* Free all my keys and values.
	 */
	void DeleteKeyValues();

	/* Answer a new BMessage that describes this entry.  The caller
	 * is reponsible for deleting the message.  The what will be
	 * ARPMSG_ENTRY.
	 */
	BMessage* AsMessage() const;
	/* Answer the tags used when writing to / reading from
	 * XML streams.
	 */
	static const char* XML_TAG()			{ return "Entry"; }	
	static const char* XML_END_TAG()		{ return "/Entry"; }
	/* These are used in the tag to identify the name and URL.
	 */
	static const char* NAME_TAG()			{ return "name"; }	
	static const char* URL_TAG()			{ return "url"; }	
	/* Write myself to an XML-formatted stream.
	 */
	ssize_t WriteTo(LpXmlStreamI& stream) const;
	ssize_t ReadFrom(LpXmlStreamI& stream);
	void ReadKeyValue(	string_vec& keys,
						string_vec& values);

	void PrintToStream() const;

	bool operator<(const LpEntry*) const;
	bool operator==(const LpEntry*) const;
	/* A sort function that can be used in, for example, STL sorts.
	 */
	static bool less_than(LpEntry* x, LpEntry* y);
	
private:
	char*		mName;
	char*		mUrl;
	string_vec	mKeys;
	string_vec	mValues;
	
	void FromMessage(const BMessage *entryMsg);
	void FreeMemory();
};
typedef vector<LpEntry*>		entry_vec;

#endif
