/* LpEntryCollection.cpp
 */
#include <algo.h>
#include <stdio.h>
#include <assert.h>
#include <string.h>
#include <malloc.h>
#include "LpEntryCollection.h"

/*************************************************************************
 * LP-ENTRY-COLLECTION
 *************************************************************************/
LpEntryCollection::LpEntryCollection()
{
}

LpEntryCollection::~LpEntryCollection()
{
	DeleteEntries();
}

const LpEntry* LpEntryCollection::EntryAt(uint32 index)
{
	if ( index >= mEntries.size() ) return 0;
	return mEntries[index];
}

void LpEntryCollection::AddEntry(LpEntry* entry)
{
	assert(entry->Name());
	LpEntry*	old_entry;
	uint32		old_index;
	if (EntryNamed(entry->Name(), &old_entry, &old_index) != B_OK) {
		mEntries.push_back(entry);
		sort( mEntries.begin(), mEntries.end(), LpEntry::less_than );
	} else {
		mEntries[old_index] = entry;
		// FIX:  I can't delete the entry that's been replaced.  It causes
		// a crash.  Why?
	}
}

status_t LpEntryCollection::RemoveEntry(const char* name)
{
	LpEntry*	entry;
	uint32		index;
	if (EntryNamed(name, &entry, &index) != B_OK) return B_ERROR;
// FIX:  I don't believe I can actually delete the object, that
// causes a crash.  WHAT THE HELL IS UP WITH THAT?
	entry_vec::iterator		i = mEntries.begin() + index;
	mEntries.erase(i);

	return B_OK;
}

status_t LpEntryCollection::EntryNamed(	const char* name,
										LpEntry** entry,
										uint32* index)
{
	assert(name != 0);
	for (uint32 k=0; k<mEntries.size(); k++) {
		if (strcmp(mEntries[k]->Name(), name) == 0) {
			*entry = mEntries[k];
			*index = k;
			return B_OK;
		}
	}
	return B_ERROR;
}

ssize_t LpEntryCollection::WriteTo(LpXmlStreamI& stream) const
{
	ssize_t		answer, total = 0;
	// Write <Entries>
	if ((answer = stream.WriteStartTag(XML_TAG())) <= 0)
		return answer;
	total += answer;
	stream.WriteCR();
	stream.PushTab();
	// Write each entry
	for (uint32 k = 0; k < mEntries.size(); k++) {
		if ( (answer = mEntries[k]->WriteTo(stream)) <= 0 ) return answer;
		total += answer;
	}
	// Write </Entries>
	stream.PopTab();
	stream.WriteTabs();
	if ((answer = stream.WriteEndTag(XML_TAG())) <= 0) return answer;
	return total + answer;

}

ssize_t LpEntryCollection::ReadFrom(LpXmlStreamI& stream)
{
	DeleteEntries();
	string_vec	keys, values;
	char*		name;
	char*		url;
	char		*tag = stream.ReadNextTag(&keys, &values);
	if (tag == 0) return 0;
	LpEntry		*entry;
	while ( (tag != 0)
			&& (strcmp( tag, XML_END_TAG() ) != 0) ) {
		if (strcmp( tag, LpEntry::XML_TAG() ) == 0) {
			if ( (GetNameAndUrl(keys, values, &name, &url) == true)
					&& ((entry = new LpEntry()) != 0) ) {
				entry->SetName(name);
				if (url != 0) entry->SetUrl(url);
				entry->ReadFrom(stream);
				AddEntry(entry);
			}
		}
		free(tag);
		FreeVector(keys);
		FreeVector(values);
		tag = stream.ReadNextTag(&keys, &values);
	}
	return 0;
}

bool LpEntryCollection::GetNameAndUrl(	string_vec& keys,
										string_vec& values,
										char** name,
										char** url)
{
	if ( (keys.size() < 1) || (values.size() < 1) ) return false;
	if ( strcmp(keys[0], LpEntry::NAME_TAG() ) == 0 )
		*name = values[0];
	*url = 0;
	if ( (keys.size() < 2) || (values.size() < 2) ) return true;
	if ( strcmp(keys[1], LpEntry::URL_TAG() ) == 0 )
		*url = values[1];
	return true;
}

void LpEntryCollection::DeleteEntries()
{
	while (mEntries.size() > 0) {
		entry_vec::iterator		i = mEntries.begin();
		delete (*i);
		mEntries.erase(i);
	}
}