/* LpEntryCollection.h
 * Copyright (c)2000 by Eric Hackborn.
 * All rights reserved.
 *
 * This code is not public domain, nor freely distributable.
 * Please direct any questions or requests to Eric Hackborn,
 * at <hackborn@angryredplanet.com>.
 *
 * ----------------------------------------------------------------------
 *
 * Known Bugs
 * ~~~~~~~~~~
 *
 *	- None!  Ha ha!
 *
 * ----------------------------------------------------------------------
 *
 * History
 * ~~~~~~~
 * 04.21.00		hackborn
 * Created this file.
 */

#ifndef LP_LPENTRYCOLLECTION_H
#define LP_LPENTRYCOLLECTION_H

#include "LpEntry.h"

/*************************************************************************
 * LP-ENTRY-COLLECTION
 * This class stores and manages LpEntry objects.  Each entry must have
 * a unique name, which is treated as the indentity of the object.
 *************************************************************************/
class LpEntryCollection
{
public:
	LpEntryCollection(); 
	virtual ~LpEntryCollection();

	/* Answer the entry at the given index, or 0 if none.
	 */
	const LpEntry* EntryAt(uint32 index);
	/* This class takes ownership of the new entry and will delete it
	 * when necessary.  If the entry matches (i.e. has the same name as)
	 * another entry in my collection, that other entry will be deleted
	 * replaced with this one.
	 */
	void AddEntry(LpEntry* entry);
	/* Remove the entry with the supplied name.  Answer B_OK if an entry
	 * was found and removed, B_ERROR otherwise.
	 */
	status_t RemoveEntry(const char* name);
	/* Find my entry with the supplied name.  Answer B_OK if I have such
	 * an entry, and if I do, place the entry in the entry arg and its
	 * index in the index arg, otherwise answer B_ERROR.
	 */
	status_t EntryNamed(const char* name,
						LpEntry** entry,
						uint32* index);

	/* Answer the tags used when writing to / reading from
	 * XML streams.
	 */
	static const char* XML_TAG()			{ return "Entries"; }	
	static const char* XML_END_TAG()		{ return "/Entries"; }
	ssize_t WriteTo(LpXmlStreamI& stream) const;
	ssize_t ReadFrom(LpXmlStreamI& stream);
	/* Extract the name and URL from the vectors and place them in the
	 * appropriate arguments.  Answer true if a name, at least, was
	 * extracted.  The url argument might be 0.
	 */
	bool GetNameAndUrl(	string_vec& keys,
						string_vec& values,
						char** name,
						char** url);

private:
	entry_vec	mEntries;

	void DeleteEntries();
};

#endif
