/* LpEntryPropertiesWindow.cpp
 */
#include <stdio.h>
#include <assert.h>
#include <string.h>
#include <malloc.h>
#include <be/interface/Button.h>
#include <be/interface/Screen.h>
#include <be/interface/StringView.h>
//#include "ArpKernel/ArpDebug.h"
#include "LpEntry.h"
#include "LpEntryPropertiesWindow.h"

#define STR_LPENTRYWINDOW		"LpEntryWindow"
static const float	BORDER_X = 5;
static const float	BORDER_Y = 5;
static const float	SPACE_X = 5;
static const float	SPACE_Y = 5;
static const float	TEXT_CTRL_Y = 20;
static const uint32	MAX_KEY_VALUE_SIZE = 5;
#define ARPMSG_OKINVOKED		'okIn'
#define ARPMSG_CANCELINVOKED	'cnIn'

static BRect figure_best_frame( BRect requested, const BWindow* caller )
{
	if( !caller ) return requested;
	
	BRect cframe = caller->Frame();
	
	BScreen s(const_cast<BWindow*>(caller));
	BRect sframe = s.Frame();
	
	// try being placed to the right.
	BRect frame(requested);
	frame.OffsetTo(cframe.right + 10, cframe.top);
	if( frame.right > sframe.right ) {
		// couldn't fit to the right; try the left.
		frame.OffsetTo(cframe.left - 10 - requested.Width(), cframe.top);
	}
	if( frame.left < cframe.left ) {
		// couldn't fit to the left; try the top.
		frame = requested;
		frame.OffsetTo(cframe.left, cframe.top - 10 - requested.Height());
	}
	if( frame.top < cframe.top ) {
		// couldn't fit to the top; try the bottom.
		frame = requested;
		frame.OffsetTo(cframe.left, cframe.bottom + 10);
	}
	if( frame.bottom > cframe.bottom ) {
		// okay, that's it.  we're out of here.
		frame = requested;
	}
	
	return frame;
}

/*************************************************************************
 * LP-ENTRY-PROPERTIES-WINDOW
 *************************************************************************/
LpEntryPropertiesWindow::LpEntryPropertiesWindow(	BRect frame,
													BWindow* caller)
		: BWindow(	figure_best_frame(frame, caller),
					STR_LPENTRYWINDOW,
					B_TITLED_WINDOW_LOOK,
					B_NORMAL_WINDOW_FEEL,
					B_NOT_ZOOMABLE | B_ASYNCHRONOUS_CONTROLS | B_NOT_RESIZABLE),
		mCaller(caller), mEntry(0), mName(0), mUrl(0)
{
	BRect		b = Bounds();
	mBg = new BView(b, "bg", B_FOLLOW_ALL, B_WILL_DRAW);
	if (mBg != 0) {
		AddChild(mBg);
		mBg->SetViewColor(ui_color(B_PANEL_BACKGROUND_COLOR));
		AddViews();
//		Pack(BORDER_X, BORDER_Y);
	}
}

LpEntryPropertiesWindow::~LpEntryPropertiesWindow()
{
}

void LpEntryPropertiesWindow::MessageReceived(BMessage *msg)
{
	switch (msg->what) {
		case ARPMSG_NEWENTRY:
			ConfigureForNew();
			break;
		case ARPMSG_ENTRY:
			ConfigureForEntry(msg);
			break;
		case ARPMSG_OKINVOKED:
			OkInvoked();
			break;
		case ARPMSG_CANCELINVOKED:
			CancelInvoked();
			break;
		default:
			inherited::MessageReceived(msg);
			break;
	}
}

void LpEntryPropertiesWindow::ConfigureForNew()
{
	SetTitle("Add Entry");
	ClearControls();
}

void LpEntryPropertiesWindow::ConfigureForEntry(BMessage* entryMsg)
{
	SetTitle("Change Entry");
	ClearControls();
	LpEntry		entry(entryMsg);
	if (mName != 0) mName->SetText( entry.Name() );
	if (mUrl != 0) mUrl->SetText( entry.Url() );
	const char*		key;
	const char*		value;
	for (uint32 k=0; entry.KeyValueAt(k, &key, &value) == B_OK; k++) {
		if ( ( k < mKeys.size() ) && ( k < mValues.size() ) ) {
			mKeys[k]->SetText(key);
			mValues[k]->SetText(value);
		}
	}
}

void LpEntryPropertiesWindow::OkInvoked()
{
	if (mCaller != 0) {
		BMessage*	msg = EntryMessage();
		if (msg != 0) {
			mCaller->PostMessage(msg);
			delete msg;
		}
	}
	PostMessage(B_QUIT_REQUESTED);
}

void LpEntryPropertiesWindow::CancelInvoked()
{
	PostMessage(B_QUIT_REQUESTED);
}

BMessage* LpEntryPropertiesWindow::EntryMessage() const
{
	LpEntry		entry;
	if ( (mName == 0) || (mName->Text() == 0) || (strlen(mName->Text()) == 0) ) return 0;
	entry.SetName( mName->Text() );
	if ( (mUrl != 0) && (mUrl->Text() != 0) && (strlen(mUrl->Text()) > 0) )
		entry.SetUrl( mUrl->Text() );
	for (uint32 k=0; k<MAX_KEY_VALUE_SIZE; k++) {
		if ( (k < mKeys.size())
				&& (k < mValues.size())
				&& (mKeys[k]->Text() != 0)
				&& (mValues[k]->Text() != 0)
				&& (strlen(mKeys[k]->Text()) > 0)
				&& (strlen(mValues[k]->Text()) > 0) ) {
			entry.AddKeyValue( mKeys[k]->Text(), mValues[k]->Text() );
		}
	}
	return entry.AsMessage();
}

void LpEntryPropertiesWindow::ClearControls()
{
	if (mName != 0) mName->SetText(0);
	if (mUrl != 0) mUrl->SetText(0);
	for (uint32 k=0; k<MAX_KEY_VALUE_SIZE; k++) {
		if ( k < mKeys.size() ) mKeys[k]->SetText(0);
		if ( k < mValues.size() ) mValues[k]->SetText(0);
	}
}

void LpEntryPropertiesWindow::AddViews()
{
	float			top = BORDER_Y;
	// Add the name and URL controls
	top = AddFullWidthTextControl(top, "Name", &mName, "text:name");
	top += BORDER_Y;
	top = AddFullWidthTextControl(top, "URL", &mUrl, "text:url");
	top += BORDER_Y;
	// Add the key and value controls
	top = AddKeyValueLabels(top);
	for (uint32 k=0; k<MAX_KEY_VALUE_SIZE; k++) {
		top = AddKeyValueControls(top);
		top += BORDER_Y;
	}
	// Add the Cancel button
	BMessage*	msg = new BMessage(ARPMSG_CANCELINVOKED);
	if( msg == 0 ) return;
	BRect		b = Bounds();
	float		cancelW = mBg->StringWidth("Cancel") + 20;
	float		btnT = top + BORDER_Y, btnR = b.right - BORDER_X - 3;
	float		btnL = btnR - cancelW, btnB = btnT + 25;
	BButton*	button = new BButton(	BRect(btnL, btnT, btnR, btnB),
										"button:Cancel",
										"Cancel",
										msg,
										B_FOLLOW_RIGHT | B_FOLLOW_TOP);
	if( button == 0 ) {
		delete msg;
		return;
	}
	mBg->AddChild(button);
	button->MakeDefault(true);
	// Add the OK button
	msg = new BMessage(ARPMSG_OKINVOKED);
	if( msg == 0 ) return;
	btnR = btnL - 10;
	btnL = btnR - cancelW;
	button = new BButton(	BRect(btnL, btnT, btnR, btnB),
							"button:OK",
							"OK",
							msg,
							B_FOLLOW_LEFT | B_FOLLOW_TOP);
	if( button == 0 ) {
		delete msg;
		return;
	}
	mBg->AddChild(button);
}

float LpEntryPropertiesWindow::AddFullWidthTextControl(	float top,
														const char* label,
														BTextControl** textCtrl,
														const char* textControlName,
														const char* initialText)
{
	BRect			b = Bounds();
	float			fontY = 10;
	BStringView*	strView;
	// Add the label
	float			labW = mBg->StringWidth(label);
	float			labL = BORDER_X, labT = top;
	float			labR = labL + labW + 5, labB = labT + fontY;
	if ( (strView = new BStringView(BRect(labL, labT, labR, labB), label, label)) != 0)
		mBg->AddChild(strView);
	top = labB;	
	// Add the text control
	float			tcL = BORDER_X - 1, tcT = top;
	float			tcR = b.right - BORDER_X, tcB = tcT + TEXT_CTRL_Y;
	if ((*textCtrl = new BTextControl(	BRect(tcL, tcT, tcR, tcB),
										textControlName,
										textControlName,
										initialText,
										0)) != 0) {
		(*textCtrl)->SetDivider(0);
		mBg->AddChild(*textCtrl);
	}
	return tcB;
}

float LpEntryPropertiesWindow::AddKeyValueLabels(float top)
{
	BRect			b = Bounds();
	float			centerX = ((b.right - b.left - BORDER_X - BORDER_X) / 2) - (SPACE_X / 2);
	float			fontY = 10;
	// Sizing for the key label
	float			keyW = mBg->StringWidth("Key");
	float			keyL = BORDER_X, keyT = top;
	float			keyR = keyL + keyW + 5, keyB = keyT + fontY;
	// Sizing for the value label
	float			valW = mBg->StringWidth("Value");
	float			valL = centerX + SPACE_X, valT = top;
	float			valR = valL + valW + 5, valB = valT + fontY;
	BStringView*	strView;
	if ( (strView = new BStringView(BRect(keyL, keyT, keyR, keyB), "Key", "Key")) != 0)
		mBg->AddChild(strView);
	if ( (strView = new BStringView(BRect(valL, valT, valR, valB), "Value", "Value")) != 0)
		mBg->AddChild(strView);
	return valB;
}

float LpEntryPropertiesWindow::AddKeyValueControls(	float top,
													const char* initialKey,
													const char* initialValue)
{
	BRect			b = Bounds();
	float			centerX = ((b.right - b.left - BORDER_X - BORDER_X) / 2) - (SPACE_X / 2);
	// Sizing for the key text control
	float			keyL = BORDER_X - 1, keyT = top;
	float			keyR = centerX, keyB = keyT + TEXT_CTRL_Y;
	// Sizing for the value text control
	float			valL = keyR + SPACE_X - 1, valT = keyT;
	float			valR = b.right - BORDER_X, valB = keyB;
	top = valB;
	BTextControl*	keyCtrl;
	BTextControl*	valCtrl;
	if ( (keyCtrl = new BTextControl(	BRect(keyL, keyT, keyR, keyB),
										"textctrl:key",
										"textctrl:key",
										initialKey,
										0)) == 0)
		return top;
	if ( (valCtrl = new BTextControl(	BRect(valL, valT, valR, valB),
										"textctrl:value",
										"textctrl:value",
										initialValue,
										0)) == 0) {
		delete keyCtrl;
		return top;
	}
	keyCtrl->SetDivider(0);
	valCtrl->SetDivider(0);
	mBg->AddChild(keyCtrl);
	mBg->AddChild(valCtrl);
	mKeys.push_back(keyCtrl);
	mValues.push_back(valCtrl);
	return top;
}

void LpEntryPropertiesWindow::Pack(float xBorder, float yBorder)
{
	if (mBg == 0) return;
	BView		*view;
	float		right = 0, bottom = 0;
	for (view = mBg->ChildAt(0); view != 0; view = view->NextSibling()) {
		BRect	b = view->Frame();
		if (b.right > right) right = b.right;
		if (b.bottom > bottom) bottom = b.bottom;		
	}
	mBg->ResizeTo(right + xBorder, bottom + yBorder);
	ResizeTo(right + xBorder, bottom + yBorder);
}

