/* LpEntryPropertiesWindow.h
 * Copyright (c)2000 by Eric Hackborn.
 * All rights reserved.
 *
 * This code is not public domain, nor freely distributable.
 * Please direct any questions or requests to Eric Hackborn,
 * at <hackborn@angryredplanet.com>.
 *
 * ----------------------------------------------------------------------
 *
 * Known Bugs
 * ~~~~~~~~~~
 *
 *	- None!  Ha ha!
 *
 * ----------------------------------------------------------------------
 *
 * History
 * ~~~~~~~
 * 04.18.00		hackborn
 * Created this file.
 */

#ifndef LP_LPENTRYPROPERTIESWINDOW_H
#define LP_LPENTRYPROPERTIESWINDOW_H

#include <be/interface/TextControl.h>
#include <be/interface/Window.h>
#include "LpEntry.h"

typedef vector<BTextControl*>		textctrl_vec;

/* Supply this message to an instance of the property window to
 * configure the property window for adding a new entry.
 */
#define ARPMSG_NEWENTRY		'newE'

/*************************************************************************
 * LP-ENTRY-PROPERTIES-WINDOW
 * This window displays and allows editing of all the properties that
 * make up an LP entry.
 *************************************************************************/
class LpEntryPropertiesWindow : public BWindow
{
public:
	LpEntryPropertiesWindow(BRect frame,
							BWindow* caller);
	virtual ~LpEntryPropertiesWindow();

	virtual void MessageReceived(BMessage *msg);

protected:
	/* The background view all of my other views get added to.
	 */
	BView			*mBg;

	/* Configure myself as a blank property window.
	 */
	void ConfigureForNew();
	/* Configure myself to edit the entry described in the supplied
	 * message.
	 */
	void ConfigureForEntry(BMessage* entryMsg);
	
	void OkInvoked();
	void CancelInvoked();
	
	/* Answer a BMessage that describes my current entry.  Caller
	 * is responsible for deleting the message.
	 */
	BMessage* EntryMessage() const;
	
private:
	typedef BWindow	inherited;

	BWindow*		mCaller;
	LpEntry*		mEntry;
	BTextControl*	mName;
	BTextControl*	mUrl;
	/* Vectors of the BTextControl objects that stores whatever keys
	 * and values have been entered for the entry I'm editing.
	 */
	textctrl_vec	mKeys;
	textctrl_vec	mValues;
	
	/* Blank out all of my controls
	 */
	void ClearControls();
	/* Build the window.
	 */	
	void AddViews();
	float AddFullWidthTextControl(	float top,
									const char* label,
									BTextControl** textCtrl,
									const char* textControlName,
									const char* initialText = 0);
	float AddKeyValueLabels(float top);
	float AddKeyValueControls(	float top,
								const char* initialKey = 0,
								const char* initialValue = 0);

	/* Resize myself to my views.
	 */
	void Pack(float xBorder, float yBorder);
};


#endif
