/* LpEntryWindow.cpp
 */
#include <stdio.h>
#include <assert.h>
#include <string.h>
#include <malloc.h>
#include <be/app/Application.h>
#include <be/app/Clipboard.h>
#include <be/app/Roster.h>
#include <be/interface/Button.h>
#include <be/interface/MenuBar.h>
#include <be/interface/MenuItem.h>
#include <be/interface/ScrollView.h>
#include <be/support/String.h>
//#include "ArpKernel/ArpDebug.h"
#include "LpApplication.h"
#include "LpEntryPropertiesWindow.h"
#include "LpEntryWindow.h"

#define ARP_SCROLLBARSIZE				(12)
#define ARP_MAINMENUSIZE				(16)

#define STR_LPENTRYWINDOW				"LpEntryWindow"

#define ARP_INDENT						(5)
#define ARP_LIST_TOP					(ARP_INDENT)
#define ARP_LIST_BOTTOM_OFFSET			((ARP_SCROLLBARSIZE *2) + 56)
#define ARP_ENTRYLIST_RIGHT				(172)

#define ARPMSG_ADDENTRY					'addE'
#define ARPMSG_REMOVEENTRY				'rmvE'
#define ARPMSG_GOWEB					'cpyU'
#define ARPMSG_COPYVALUE				'pstV'
#define ARPMSG_ENTRYSELECTED			'iEnS'
#define ARPMSG_ENTRYINVOKED				'iEnI'

#define ARPSTR_ADD_ENTRY_NAME			"button:AddEntry"
#define ARPSTR_CHANGE_ENTRY_NAME		"button:ChangeEntry"
#define ARPSTR_REMOVE_ENTRY_NAME		"button:RemoveEntry"
#define ARPSTR_BUTTON					"button"

static const char*	URL_BUTTON_NAME		= "button:copyurl";
static float		paste_button_W		= 0;
static float		paste_button_T		= 0;
static float		button_y			= 24;
static const float	SPACE_X				= 5;
static const float	SPACE_Y				= 5;

/*************************************************************************
 * LP-ENTRY-WINDOW
 *************************************************************************/
LpEntryWindow::LpEntryWindow(BRect frame)
		: inherited(frame,
					STR_LPENTRYWINDOW,
					B_DOCUMENT_WINDOW_LOOK,
					B_NORMAL_WINDOW_FEEL,
					B_NOT_ZOOMABLE | B_WILL_ACCEPT_FIRST_CLICK),
		mEntries(0), mEntryView(0), mEntryString(0)
{
	mEntries = new LpEntryCollection();
	SetTitle("Lesser Passwords");
	BRect		b = Bounds();
	mBg = new BView(BRect(b.left, b.top+17, b.right, b.bottom-14), "bg", B_FOLLOW_ALL, B_WILL_DRAW);
	if (mBg != 0) {
		AddMainMenu();
		BPoint		pos, tmp;
		float		top;
		float		min_h = 0, max_h = 100000, min_v = 133, max_v = 100000;
		AddChild(mBg);
		mBg->SetViewColor(ui_color(B_PANEL_BACKGROUND_COLOR));
		pos = AddEntryList();
		min_h = ( AddButtonLayer(BPoint(ARP_INDENT, pos.y + ARP_INDENT)) ).x;
		AddMessageLayer();
		pos.Set(ARP_ENTRYLIST_RIGHT + 15 + 5, 5);
		top = AddLabel(pos, "Go to website");
		pos.y = top - 2;
		paste_button_W = b.right - SPACE_X - pos.x;
		tmp = AddButton(pos,
						ARPMSG_GOWEB,
						URL_BUTTON_NAME,
						"URL",
						b.right - SPACE_X,
						B_FOLLOW_RIGHT | B_FOLLOW_TOP);
		pos.y = tmp.y + SPACE_Y + SPACE_Y;
		top = AddLabel(pos, "Copy to paste buffer");
		paste_button_T = top;
		// Set the state.	
		EntrySelected();

		SetSizeLimits(min_h, max_h, min_v, max_v);
	}
}

LpEntryWindow::~LpEntryWindow()
{
	DeleteEntryItems();
	delete mEntries;
}

void LpEntryWindow::AddMainMenu()
{
	BMenuBar	*menuBar;
	BMenu		*menu;
	BMenuItem	*menuItem;
	BRect		rect = Bounds();
	
	rect.right++;
	rect.bottom = rect.top + ARP_MAINMENUSIZE;
	menuBar = new BMenuBar(rect, NULL, B_FOLLOW_TOP | B_FOLLOW_LEFT_RIGHT,
			B_ITEMS_IN_ROW, FALSE);
	if (menuBar == 0) return;
	/* File Menu
	 */
	menu = new BMenu("File", B_ITEMS_IN_COLUMN);
	AddMenuItem(menu, "About", B_ABOUT_REQUESTED, 'A');
	menu->AddSeparatorItem();
	AddMenuItem(menu, "Quit", B_QUIT_REQUESTED, 'Q');
	menuItem = new BMenuItem(menu);
	menuBar->AddItem(menuItem);
#if 0
	/* EDIT Menu
	 */
	menu = new BMenu(STR_EDIT, B_ITEMS_IN_COLUMN);
//	AddMenuItem(menu, "Cut", ARPMSG_CUT, 'X');
	AddMenuItem(menu, "Copy", ARPMSG_COPY, 'C');
	AddMenuItem(menu, "Paste", ARPMSG_PASTE, 'V');
	menu->AddSeparatorItem();
	AddMenuItem(menu, "Patch", ARPMSG_PATCHINVOKED, 'I');
	menuItem = new BMenuItem(menu);
	menuBar->AddItem(menuItem, EDIT_MENU_INDEX);
#endif
	/* Menu Bar
	 */
//	mBg->AddChild(menuBar);
	AddChild(menuBar);
	SetKeyMenuBar(menuBar);
}

BPoint LpEntryWindow::AddEntryList()
{
	float		labelHeight = 10;
	BRect		b = Bounds();
	BPoint		answer = BPoint(ARP_ENTRYLIST_RIGHT,
								b.bottom - ARP_LIST_BOTTOM_OFFSET + ARP_SCROLLBARSIZE + 1);
	float		listL = ARP_INDENT, listT = ARP_LIST_TOP + labelHeight +1;
	float		listR = ARP_ENTRYLIST_RIGHT, listB = b.bottom - ARP_LIST_BOTTOM_OFFSET - labelHeight - 1;
	BListView	*listView = new BListView(	BRect(listL, listT, listR, listB),
											"EntryList",
											B_SINGLE_SELECTION_LIST,
											B_FOLLOW_ALL);
	if (listView == 0) return answer;
	BScrollView	*scrollView = new BScrollView("ScrollEntryList", listView,
											  B_FOLLOW_ALL,
											  0, true, true);
	if (scrollView == 0) {
		delete listView;
		return answer;
	}
	scrollView->SetViewColor(ui_color(B_PANEL_BACKGROUND_COLOR));
	mBg->AddChild(scrollView);
	mEntryView = listView;
	listView->SetSelectionMessage(new BMessage(ARPMSG_ENTRYSELECTED));
	listView->SetInvocationMessage(new BMessage(ARPMSG_ENTRYINVOKED));

	// Add a little box that will obscure the undrawn portion of the list view.
	BView	*plug = new BView(	BRect(listR + 2, listB + 2, listR + 15, listB + 15),
								"entry plug", B_FOLLOW_RIGHT | B_FOLLOW_BOTTOM, 0);
	if (plug != 0) {
		plug->SetViewColor(ui_color(B_PANEL_BACKGROUND_COLOR));
		mBg->AddChild(plug);
	}

	// Add a name label for the bank list
	BStringView		*strView = new BStringView(BRect(ARP_INDENT,
													 ARP_LIST_TOP,
													 ARP_ENTRYLIST_RIGHT,
													 ARP_LIST_TOP + labelHeight),
												"Entries",
												"Entries");
	if (strView != 0) {
		strView->SetViewColor(ui_color(B_PANEL_BACKGROUND_COLOR));
		mBg->AddChild(strView);
	}
	// Add an item / selection label for the bank list
	float	labL = ARP_INDENT, labT = listB + 15 + 6;
	float	labR = ARP_ENTRYLIST_RIGHT, labB = labT + labelHeight;
	mEntryString = new BStringView(	BRect(labL, labT, labR, labB),
									"Entry string",
									"0 entries",
									B_FOLLOW_LEFT | B_FOLLOW_BOTTOM);
	if (mEntryString != 0) {
		mEntryString->SetViewColor(ui_color(B_PANEL_BACKGROUND_COLOR));
		mBg->AddChild(mEntryString);
	}
	answer.y = labB;
	return answer;
}

BPoint LpEntryWindow::AddButtonLayer(BPoint topLeft)
{
	BPoint	pt;
	float	removeW = mBg->StringWidth("Remove"), changeW = mBg->StringWidth("Change");
	float	btnW = (removeW > changeW) ? removeW : changeW;
	btnW += 20;
	
	pt = AddButton(	topLeft,
					ARPMSG_ADDENTRY,
					ARPSTR_ADD_ENTRY_NAME,
					"Add",
					topLeft.x + btnW);
	pt = AddButton(	BPoint(pt.x + ARP_INDENT, topLeft.y),
					ARPMSG_ENTRYINVOKED,
					ARPSTR_CHANGE_ENTRY_NAME,
					"Change",
					pt.x + ARP_INDENT + btnW);
	pt = AddButton(	BPoint(pt.x + ARP_INDENT, topLeft.y),
					ARPMSG_REMOVEENTRY,
					ARPSTR_REMOVE_ENTRY_NAME,
					"Remove",
					pt.x + ARP_INDENT + btnW);
	return pt;
}

BPoint LpEntryWindow::AddButton(BPoint topLeft,
								uint32 messageWhat,
								const char* name,
								const char* label,
								float right,
								uint32 resizeMask)
{
	BMessage*	msg = new BMessage(messageWhat);
	if( msg == 0 ) return topLeft;
	float		labelW = mBg->StringWidth(label);
	float		btnR = topLeft.x + labelW + 10;
	if (right != -1) btnR = right;
	BRect		frame(	topLeft.x,
						topLeft.y,
						btnR,
						topLeft.y + button_y);
	BButton*	button = new BButton(	frame,
										name,
										label,
										msg,
										resizeMask);
	if( button == 0 ) {
		delete msg;
		return topLeft;
	}
	msg->AddPointer(ARPSTR_BUTTON, button);
	mBg->AddChild(button);
	return BPoint(frame.right, frame.bottom);
}

float LpEntryWindow::AddLabel(BPoint topLeft, const char* text)
{
	float			fontY = 10;
	// Sizing for the key label
	float			cpyW = mBg->StringWidth(text);
	float			labL = topLeft.x, labT = topLeft.y;
	float			labR = labL + cpyW + 5, labB = labT + fontY;
	BStringView*	strView;
	if ( (strView = new BStringView(BRect(labL, labT, labR, labB), "label", text, B_FOLLOW_RIGHT | B_FOLLOW_TOP)) != 0)
		mBg->AddChild(strView);
	return labB;
}

void LpEntryWindow::AddMessageLayer()
{
	BRect			b = Bounds();
	BRect			r(b.left, b.bottom - 13, b.right - 14, b.bottom + 1);
	BView*			plug = new BView(r, "plug", B_FOLLOW_LEFT_RIGHT | B_FOLLOW_BOTTOM, B_WILL_DRAW);
	if (plug != 0) {
		plug->SetViewColor(ui_color(B_PANEL_BACKGROUND_COLOR));
		AddChild(plug);
	}
}

bool LpEntryWindow::QuitRequested()
{
	LpApplication*	app = dynamic_cast<LpApplication*>(be_app);
	if ( (app != 0) && (app->IsQuitting() == false) ) {
		be_app->PostMessage(B_QUIT_REQUESTED);
		return false;
	}

	return true;
}

void LpEntryWindow::MessageReceived(BMessage *msg)
{
	switch (msg->what) {
		case B_ABOUT_REQUESTED:
			be_app->PostMessage(B_ABOUT_REQUESTED);
			break;
		case ARPMSG_ENTRY:
			AddEntry(msg);
			break;
		case ARPMSG_ADDENTRY:
			AddEntry();
			break;
		case ARPMSG_GOWEB:
			GoWeb();
			break;
		case ARPMSG_COPYVALUE:
			const char*	value;
			if (msg->FindString("value", &value) == B_OK)
				CopyString(value);
			break;
		case ARPMSG_REMOVEENTRY:
			RemoveEntry();
			break;
		case ARPMSG_ENTRYSELECTED:
			EntrySelected();
			break;
		case ARPMSG_ENTRYINVOKED:
			EntryInvoked();
			break;
		default:
			inherited::MessageReceived(msg);
			break;
	}
}

ssize_t LpEntryWindow::WriteState(LpXmlStreamI& stream)
{
	// Write <Window type = LpEntryWindow>
	string_vec		keys, values;
	keys.push_back(ARPSTR_TYPE);
	values.push_back(ARPSTR_LPENTRYWINDOW);
	if (stream.WriteStartTag(ARPSTR_WINDOW_XML_TAG, &keys, &values) <= 0) return B_ERROR;
	stream.WriteCR();
	stream.PushTab();
	// Write <frame>leftScale topScale rightScale bottomScale</frame>
	WriteFrame(&stream);
	// Write the entries
	if (mEntries != 0) mEntries->WriteTo(stream);
	// Write </Window>
	stream.PopTab();
	stream.WriteTabs();
	if (stream.WriteEndTag(ARPSTR_WINDOW_XML_TAG) <= 0) return B_ERROR;
	return 0;
}

ssize_t LpEntryWindow::ReadState(	LpXmlStreamI& stream,
									const char* windowTag,
									const char* windowType)
{
	char			*frameStr = 0;
	char			*tag;
	while ( ((tag = stream.ReadNextTag()) != 0)
			&& (strcmp(tag, ARPSTR_WINDOW_XML_TAG) != 0) ) {
		if (strcmp( tag, inherited::FrameTag() ) == 0) {
			frameStr = stream.ReadStrToTag("frame");
		} else if (strcmp( tag, LpEntryCollection::XML_TAG() ) == 0) {
			if (mEntries == 0) mEntries = new LpEntryCollection();
			if (mEntries != 0) mEntries->ReadFrom(stream);
			SynchronizeEntryItems();
		}
		free(tag);
	}
	// Set my frame
	if (frameStr != 0) {
		ReadFrame(frameStr);
		free(frameStr);
	}
	return 0;
}

void LpEntryWindow::AddEntry()
{
	BMessage	msg(ARPMSG_NEWENTRY);
	ShowPropertiesWin(&msg);
}

void LpEntryWindow::AddEntry(BMessage* entryMsg)
{
	LpEntry		*entry = new LpEntry(entryMsg);
	if (entry == 0) return;
	if (entry->Name() == 0) return;
	if (mEntries != 0) {
		mEntries->AddEntry(entry);
		SynchronizeEntryItems();
		LpEntryItem*	item = EntryItemNamed( entry->Name() );
		if (item != 0) {
			int32	index = EntryView()->IndexOf(item);
			if (index >= 0) {
				// The ListView is efficient and won't reselect the item if
				// it's already been selected -- however, I WANT to get the
				// EntrySelected() method, one way or another.
				if ( item->IsSelected() ) EntrySelected();
				else EntryView()->Select(index);
			}
		}
	}
}

void LpEntryWindow::RemoveEntry()
{
	if( mEntries == 0 ) return;
	LpEntryItem*	item = CurrentEntryItem();
	if( item == 0 ) return;
	item->RemoveFrom(mEntries);
	SynchronizeEntryItems();
	EntrySelected();
}

void LpEntryWindow::GoWeb() const
{
	LpEntry*		entry = CurrentEntry();
	if (entry == 0) return;
	if (entry->Url() == 0) return;

	char* argv[2];
	argv[0] = (char*)entry->Url();
	argv[1] = 0;
	be_roster->Launch("application/x-vnd.Be.URL.http", 1, argv);
}

void LpEntryWindow::CopyString(const char* string) const
{
	assert(string != 0);
	if (be_clipboard->Lock()) {
		BMessage*	clip;
		size_t		size = strlen( string );
		be_clipboard->Clear();
		if ((clip = be_clipboard->Data()) != 0) {
			clip->AddData("text/plain", B_MIME_TYPE, string, size);
			be_clipboard->Commit();
		}
		be_clipboard->Unlock();
	}
}

void LpEntryWindow::ShowPropertiesWin(BMessage* msg)
{
	if( !mPropertiesWin.IsValid() ) {
		LpEntryPropertiesWindow	*win = new LpEntryPropertiesWindow(BRect(100, 100, 250, 350), this);
		mPropertiesWin = BMessenger(win);
		if (msg != 0) win->PostMessage(msg);
		win->Show();
	} else {
		BHandler	*target;
		BLooper		*looper;
		if ((target = mPropertiesWin.Target(&looper)) != 0) {
			if (msg != 0) ((BWindow*)target)->PostMessage(msg);
			((BWindow*)target)->Activate(true);
		}
	}
}

void LpEntryWindow::EntrySelected()
{
	SetEntryString();
	if (mBg == 0) return;
	bool	selections = false;
	if ( (EntryView() != 0)
			&& (EntryView()->CurrentSelection() >= 0) )
		selections = true;
	
	BButton	*button = dynamic_cast<BButton*>(mBg->FindView(ARPSTR_REMOVE_ENTRY_NAME));
	if (button != 0) button->SetEnabled(selections);
	button = dynamic_cast<BButton*>(mBg->FindView(ARPSTR_CHANGE_ENTRY_NAME));
	if (button != 0) button->SetEnabled(selections);

	SetVisibleProperties( CurrentEntry() );
}

void LpEntryWindow::SetVisibleProperties(LpEntry* entry)
{
	BButton*	button = dynamic_cast<BButton*>(mBg->FindView(URL_BUTTON_NAME));
	if (button != 0) {
		if ( (entry != 0) && (entry->Url() != 0) ) {
			SetButtonLabel( button, entry->Url() );
			button->SetEnabled(true);
		} else {
			SetButtonLabel( button, "No URL" );
			button->SetEnabled(false);
		}
	}

	const char*	key;
	const char*	value;
	uint32	k;
	for (k=0; ((entry != 0) && (entry->KeyValueAt(k, &key, &value) == B_OK)); k++) {
		if ( k<mPasteButtons.size() ) SetPasteButton(mPasteButtons[k], key, value);
		else AddPasteButton(k, key, value);
	}
	while ( k<mPasteButtons.size() ) {
		if ( !(mPasteButtons[k]->IsHidden()) ) mPasteButtons[k]->Hide();
		k++;
	}
}

void LpEntryWindow::SetPasteButton(	BButton* button,
									const char* key,
									const char* value)
{
	assert(button != 0);
	SetButtonLabel( button, key );
	BMessage	*msg = new BMessage(ARPMSG_COPYVALUE);
	if (msg != 0) msg->AddString("value", value);
	button->SetMessage(msg);
	if (button->Message() == 0) button->SetEnabled(false);
	else button->SetEnabled(true);
	if ( button->IsHidden() ) button->Show();
}

void LpEntryWindow::AddPasteButton(	uint32 index,
									const char* key,
									const char* value)
{
	BMessage*	msg = new BMessage(ARPMSG_COPYVALUE);
	if( msg == 0 ) return;
	msg->AddString("value", value);
	BRect		b = Bounds();
	float		btnT = paste_button_T + ( index * (button_y + SPACE_Y));
	float		btnR = b.right - SPACE_X;
	float		btnB = btnT + button_y;
	float		btnL = btnR - paste_button_W;
	BButton*	button = new BButton(	BRect(btnL, btnT, btnR, btnB),
										"button:value",
										key,
										msg,
										B_FOLLOW_RIGHT | B_FOLLOW_TOP);
	if( button == 0 ) {
		delete msg;
		return;
	}
	SetButtonLabel( button, key );
	mPasteButtons.push_back(button);
	mBg->AddChild(button);
}

void LpEntryWindow::SetButtonLabel(BButton* button, const char* label)
{
	if (button == 0) return;
	if (label == 0) return;
	BString	bstr(label);
	button->TruncateString( &bstr, 0, button->Bounds().Width() - 8 );
	button->SetLabel( bstr.String() );
}

void LpEntryWindow::EntryInvoked()
{
	LpEntry*	entry = CurrentEntry();
	if (entry == 0) return;
	BMessage*	msg = entry->AsMessage();
	if (msg == 0) return;
	ShowPropertiesWin(msg);
	delete msg;
}

LpEntry* LpEntryWindow::CurrentEntry() const
{
	if (mEntries == 0) return 0;
	LpEntryItem*	item = CurrentEntryItem();
	if (item == 0) return 0;
	LpEntry*	entry;
	uint32		index;
	if (mEntries->EntryNamed(item->Text(), &entry, &index) != B_OK) return 0;
	return entry;
}

LpEntryItem* LpEntryWindow::CurrentEntryItem() const
{
	if (EntryView() == 0) return 0;
	int32	index = EntryView()->CurrentSelection();
	if (index < 0) return 0;
	LpEntryItem*	item = dynamic_cast<LpEntryItem*>(EntryView()->ItemAt(index));
	if (item == 0) return 0;
	return item;	
}

LpEntryItem* LpEntryWindow::EntryItemNamed(const char* name) const
{
	if (EntryView() == 0) return 0;
	LpEntryItem*	item;
	for (int32 k=0; (item = dynamic_cast<LpEntryItem*>(EntryView()->ItemAt(k)) ) != 0; k++) {
		if (strcmp(item->Text(), name) == 0) return item;
	}
	return 0;
}

void LpEntryWindow::SetEntryString()
{
	if (mEntryString == 0) return;
	if (EntryView() == 0) {
		mEntryString->SetText("0 items");
		return;
	}
	int32		count = EntryView()->CountItems();
	const char*	category = "item";
	char		str[32];
	if (count == 1) sprintf(str, "1 %s", category);
	else sprintf(str, "%ld %ss", count, category);
	mEntryString->SetText(str);
}

void LpEntryWindow::SynchronizeEntryItems()
{
	if (EntryView() == 0) return;
	if (mEntries == 0) return;
	
	const LpEntry*	entry;
	LpEntryItem*	item;
	uint32			k;
	for (k=0; (entry = mEntries->EntryAt(k)) != 0; k++) {
		if ( (item = dynamic_cast<LpEntryItem*>(EntryView()->ItemAt(k))) != 0) {
			item->Synchronize(entry);
		} else {
			item = new LpEntryItem(entry);
			if (item != 0) EntryView()->AddItem(item);
		}
	}
	while ( EntryView()->ItemAt(k) != 0) {
		BListItem*	listItem = EntryView()->RemoveItem( (int32)k );
		delete listItem;
	}

	EntryView()->Invalidate();
	SetEntryString();
}

void LpEntryWindow::DeleteEntryItems()
{
	if (EntryView() != 0) {
		BListItem	*item;
		while ( (item = EntryView()->RemoveItem((int32)0)) != 0)
			delete item;
	}
}
