/* LpEntryWindow.h
 * Copyright (c)2000 by Eric Hackborn.
 * All rights reserved.
 *
 * This is the main window for an ExDeviceI.  It allows the
 * user to browse the various types of data in the device, manage
 * this data, and open up editors on the data types.
 *
 * This code is not public domain, nor freely distributable.
 * Please direct any questions or requests to Eric Hackborn,
 * at <hackborn@angryredplanet.com>.
 *
 * ----------------------------------------------------------------------
 *
 * Known Bugs
 * ~~~~~~~~~~
 *
 *	- None!  Ha ha!
 *
 * ----------------------------------------------------------------------
 *
 * History
 * ~~~~~~~
 * 04.18.00		hackborn
 * Created this file.
 */

#ifndef LP_LPENTRYWINDOW_H
#define LP_LPENTRYWINDOW_H

#include <be/interface/ListView.h>
#include <be/interface/StringView.h>
#include <be/storage/FilePanel.h>
#include <be/support/Locker.h>
#include "LpEntryCollection.h"
#include "LpEntryItem.h"
#include "LpWindow.h"
#include "LpWindowStateI.h"

typedef vector<BButton*>		button_vec;

/*************************************************************************
 * LP-ENTRY-WINDOW
 *************************************************************************/
#define ARPSTR_LPENTRYWINDOW	"LpEntryWindow"

class LpEntryWindow : public LpWindow,
					  public LpWindowStateI
{
public:
	LpEntryWindow(BRect frame); 
	virtual ~LpEntryWindow();

	virtual	bool QuitRequested();
	virtual void MessageReceived(BMessage *msg);

	virtual ssize_t WriteState(	LpXmlStreamI& stream);
	virtual ssize_t ReadState(	LpXmlStreamI& stream,
								const char* windowTag,
								const char* windowType);

protected:
	/* The background view all of my other views get added to.
	 */
	BView			*mBg;
	
	void AddEntry();
	void AddEntry(BMessage* entryMsg);
	void RemoveEntry();
	void GoWeb() const;
	void CopyString(const char* string) const;
	void ShowPropertiesWin(BMessage* msg);
	
private:
	typedef LpWindow	inherited;
	LpEntryCollection	*mEntries;
	/* A messenger to the window that displays the properties for
	 * a single Entry.
	 */
	BMessenger 			mPropertiesWin;
	/* A vector of all the buttons that can be used to copy to the
	 * paste buffer.  There will be one button for each key / value
	 * pair in the current entry.
	 */
	button_vec			mPasteButtons;
	
	void AddMainMenu();
	BPoint AddEntryList();
	BPoint AddButtonLayer(BPoint topLeft);
	BPoint AddButton(	BPoint topLeft,
						uint32 messageWhat,
						const char* name,
						const char* label,
						float right = -1,
						uint32 resizeMask = B_FOLLOW_LEFT | B_FOLLOW_BOTTOM);
	float AddLabel(BPoint topLeft, const char* text);
	void AddMessageLayer();

	BListView*		mEntryView;
	BListView* EntryView() const 	{ return mEntryView; }
	void EntrySelected();
	/* Set the state of the buttons in the right-hand pane.
	 */
	void SetVisibleProperties(LpEntry* entry);
	/* Set the paste button up appropriately according to the key and value.
	 */
	void SetPasteButton(BButton* button,
						const char* key,
						const char* value);
	/* Add a button for one of the key / value pairs in an entry.
	 */
	void AddPasteButton(uint32 index,
						const char* key,
						const char* value);
	/* Set the supplied button's label to label, truncated if necessary.
	 */
	void SetButtonLabel(BButton* button, const char* label);
	void EntryInvoked();
	LpEntry* CurrentEntry() const;
	LpEntryItem* CurrentEntryItem() const;
	LpEntryItem* EntryItemNamed(const char* name) const;
	
	BStringView*	mEntryString;
	void SetEntryString();

	/* Make all the entry items match what's currently in the
	 * entry collection.
	 */
	void SynchronizeEntryItems();
	/* Delete all the list items stored in the entry list view.
	 */
	void DeleteEntryItems();
};


#endif
