/* LpWindow.cpp
 */
#include <stdio.h>
#include <assert.h>
#include <string.h>
#include <malloc.h>
#include <be/interface/Menu.h>
#include <be/interface/MenuItem.h>
#include <be/interface/Screen.h>
#include "LpWindow.h"

/*************************************************************************
 * LP-WINDOW
 *************************************************************************/
LpWindow::LpWindow(	BRect frame,
					const char* title, 
					window_look look,
					window_feel feel,
					uint32 flags,
					uint32 workspace)
		: BWindow(frame, title, look, feel, flags, workspace)
{
}

status_t LpWindow:: ReadFrame(const char* frameStr)
{
	assert(frameStr != 0);
	BScreen 	screen(this);
	BRect		frame(-1, -1, -1, -1), sf = screen.Frame();
	size_t		len = strlen(frameStr);
	size_t		index = 0;
	char*		str;
	bool		minimized = false;
	
	if ((str = NextNumber(frameStr, len, &index)) == 0) return B_ERROR;
	frame.left = atof(str) * sf.Width();
	free(str);
	if ((str = NextNumber(frameStr, len, &index)) == 0) return B_ERROR;
	frame.top = atof(str) * sf.Height();
	free(str);
	if ((str = NextNumber(frameStr, len, &index)) == 0) return B_ERROR;
	frame.right = atof(str) * sf.Width();
	free(str);
	if ((str = NextNumber(frameStr, len, &index)) == 0) return B_ERROR;
	frame.bottom = atof(str) * sf.Height();
	free(str);
	// Get the minimized flag.  If there isn't one, that's not an error
	// condition.
	if ((str = NextNumber(frameStr, len, &index)) != 0) {
		if (atol(str) == 1) minimized = true;
		free(str);
	}

	Minimize(minimized);
	MoveTo(frame.left, frame.top);
	ResizeTo(frame.Width(), frame.Height());
	return B_OK;
}

char* LpWindow::NextNumber( const char* str,
							size_t len,
							size_t *index)
{
	size_t	start = *index;
	while (IsWhiteSpace(str[start])) {
		start++;
		if (start >= len) return 0;
	}
	*index = start;
	while ( ((*index + 1) < len)
			&& (IsWhiteSpace(str[*index + 1]) == false) )
		*index = *index + 1;
	char	*substr = (char*)malloc(sizeof(char) * ((*index - start) + 5));
	if (substr == 0) return 0;
	size_t	i = 0;
	for (size_t k=start; k<=(*index); k++) {
		substr[i] = str[k];
		i++;
	}
	substr[i] = 0;
	*index = *index + 1;
	return substr;
}

bool LpWindow::IsWhiteSpace(char c)
{
	if (c == 9) return true;	// Tab
	if (c == 13) return true;	// CR
	if (c == 32) return true;	// Space
	return false;
}

status_t LpWindow::WriteFrame(LpXmlStreamI* stream)
{
	BScreen 	screen(this);
	BRect		f = Frame(), sf = screen.Frame();
	char		buf[64];
	uint8		minimized = 0;
	if (IsMinimized()) minimized = 1;
	sprintf(buf, "%3.8f %3.8f %3.8f %3.8f %d",
						f.left / sf.Width(),
						f.top / sf.Height(),
						f.right / sf.Width(),
						f.bottom / sf.Height(),
						minimized);
	stream->WriteStartTag( LpWindow::FrameTag() );
	stream->Write( buf, strlen(buf) );
	stream->WriteEndTag( LpWindow::FrameTag() );
	return B_OK;
}

status_t LpWindow::AddMenuItem(	BMenu* toMenu,
								const char* label,
								uint32 what,
								char shortcut)
{
	BMessage*	msg = new BMessage(what);
	if (msg == 0) return B_NO_MEMORY;
	return AddMenuItem(toMenu, label, msg, shortcut);
}

status_t LpWindow::AddMenuItem(	BMenu* toMenu,
								const char* label,
								BMessage* msg,
								char shortcut)
{
	BMenuItem*	item = new BMenuItem(label, msg, shortcut);
	if (item == 0) {
		delete msg;
		return B_NO_MEMORY;
	}
	toMenu->AddItem(item);
	return B_OK;
}