/* LpWindow.h
 * Copyright (c)2000 by Eric Hackborn.
 * All rights reserved.
 *
 * This code is not public domain, nor freely distributable.
 * Please direct any questions or requests to Eric Hackborn,
 * at <hackborn@angryredplanet.com>.
 *
 * ----------------------------------------------------------------------
 *
 * Known Bugs
 * ~~~~~~~~~~
 *
 *	- None!  Ha ha!
 *
 * ----------------------------------------------------------------------
 *
 * History
 * ~~~~~~~
 * 04.18.00		hackborn
 * Created this file.
 */

#ifndef LP_LPWINDOW_H
#define LP_LPWINDOW_H

#include <be/interface/Window.h>
#include "LpXmlStreamI.h"

/***************************************************************************
 * EX-WINDOW
 * A commonly used ARP abstract superclass for windows.  Contains simple
 * behaviour like reading and writing its frame to an XML stream and
 * conveniences for adding menu items.
 ***************************************************************************/
class LpWindow : public BWindow
{
protected:
	LpWindow(	BRect frame,
				const char* title, 
				window_look look,
				window_feel feel,
				uint32 flags,
				uint32 workspace = B_CURRENT_WORKSPACE);

	/* Write my frame information out to the supplied stream.  The result
	 * is an entry in the stream that looks something like:
	 *		<frame> 0.243345 0.02943 0.459454 0.34545</frame>
	 * where the numbers are the left edge, top edge, right edge, and bottom
	 * edge of the window, respectively.  The numbers will be between 0 and
	 * 1.
	 */
	status_t WriteFrame(LpXmlStreamI* stream);
	/* Read my frame from the supplied string.  The string should be in the
	 * same format as WriteFrame() created -- i.e., a series of four numbers
	 * between 0 and 1.  Each number is treated as a percentage of the width
	 * or height of the screen.  For example, if the first number is 0.5, the
	 * result is to put the left edge at exactly the middle of the screen.
	 */
	status_t ReadFrame(const char* frameStr);

	static const char* FrameTag()			{ return "frame"; }

	/* The following is a convenience for creating a standard menu item
	 * with the supplied info.  Either B_NO_MEMORY or B_OK will be answered.
	 */
	status_t AddMenuItem(	BMenu* toMenu,
							const char* label,
							uint32 what,
							char shortcut);
	status_t AddMenuItem(	BMenu* toMenu,
							const char* label,
							BMessage* msg,
							char shortcut);

private:
	char* NextNumber(	const char* str,
						size_t len,
						size_t* index);
	bool IsWhiteSpace(char c);
};

#endif
