/* LpXmlStreamI.h
 * Copyright (c)2000 by Eric Hackborn.
 * All rights reserved.
 *
 * This code is not public domain, nor freely distributable.
 * Please direct any questions or requests to Eric Hackborn,
 * at <hackborn@angryredplanet.com>.
 *
 * ----------------------------------------------------------------------
 *
 * To Do
 * ~~~~~~~~~~
 *
 *	- Nothing!
 *
 * ----------------------------------------------------------------------
 *
 * Known Bugs
 * ~~~~~~~~~~
 *
 *	- None!  Ha ha!
 *
 * ----------------------------------------------------------------------
 *
 * History
 * ~~~~~~~
 * 04.18.00		hackborn
 * Created this file.
 */

#ifndef LP_LPXMLSTREAMI_H
#define LP_LPXMLSTREAMI_H

#include <be/support/DataIO.h>
#include "string_vec.h"

/***************************************************************************
 * LP-XML-STREAM-I
 * This class defines the interface for the XML stream that ExFiler classes
 * are written to and read from.
 ***************************************************************************/
class LpXmlStreamI
{
public:
	virtual ~LpXmlStreamI()	{ }

	virtual void PushTab() = 0;
	virtual void PopTab() = 0;

	/* Answer the next tag in the stream.  The client owns the data
	 * returned, so they must dispose of it.  If the client is expecting
	 * the tag to have keys and values, it can supply vectors to store
	 * those objects.  Note, though, that whatever data is placed in the
	 * vectors must be freed, regardless of whether or not the data is
	 * correct.
	 */
	virtual char* ReadNextTag(	string_vec *keys = 0,
								string_vec *values = 0) = 0;

	/* Answer, as a string, all my contents from my current position
	 * up until the tag specified.  The client is the owner of the
	 * returned string, so is responsible for deleting it.
	 */
	virtual char* ReadStrToTag(const char *endTag) = 0;

	/* Each Write() function answers the number of bytes written,
	 * or a negative error value.  Optionally, string_vec objects
	 * containing keys and values can be added.  These keys and values
	 * appear in the tag in the form " [key] = [value]".  This means
	 * that tags, keys, and values cannot have spaces in them, as the
	 * space is used as a delimiter.
	 */	
	virtual ssize_t WriteStartTag(	const char *tag,
									string_vec *keys = 0,
									string_vec *values = 0) = 0;
	virtual ssize_t WriteEndTag(const char *tag) = 0;
	virtual ssize_t Write(const void *buffer, size_t size) = 0;
	virtual ssize_t WriteChar(char c) = 0;
	virtual ssize_t WriteCR() = 0;
	virtual ssize_t WriteSpace() = 0;
	/* Write a tab character for however many tabs have been
	 * pushed on me.
	 */
	virtual ssize_t WriteTabs() = 0;

	/* Return the version of the addon interface that this
	 * object is implementing.  Addons should define this
	 * to always return VERSION_CURRENT; the MIDI system will
	 * use this to make sure it doesn't try to make your addon
	 * do anything it can't.
	 */
	enum {
		VERSION_1 = 19991013,	// First version
			
		VERSION_CURRENT = VERSION_1
	};
	typedef uint32 VersionType;
	
	virtual VersionType Version(void) const = 0;
};

#endif
